'     PROGRAM TO REAGGREGATE FROM PYCNOPHYLACTICALLY REALLOCATED DATA
      OPTION BASE 1
      DEFSNG A-H, O-Z: DEFINT I-N
'
'     ORIGINAL PC VERSION AUGUST 1992
'     MS QuickBasic 4.5 from original Fortran program
' 
'     Copyright 1992 Waldo Tobler
'     *****************************************************************
'
'     PROFESSOR WALDO TOBLER
'     DEPARTMENT OF GEOGRAPHY
'     UNIVERSITY OF CALIFORNIA
'     SANTA BARBARA, CALIFORNIA
'     93106-4060  USA
'     tobler@geog.ucsb.edu
'
'     *****************************************************************
'
'     Reaggregation using an interpolated array.
'     For use with census-type data.
'     You need two identically sized arrays, one with zone labels,
'     the other with numerical values.
'     Nr = Number of regions or zones
'     Rw%, Cl% define the size of the arrays
'     IMPORTANT: The outside of the domain of interest must have as its ID the value 0 (zero)
'     Version 001 of the program (Microsoft QuickBasic 4.5)
      CLS
      PRINT " REAGGREGATION FROM PYCHNOPHYLACTIC INTERPOLATION, BY W.TOBLER"
      PRINT
      INPUT " Enter the name of the file containing the ID labels "; ID$
      PRINT
      OPEN ID$ FOR INPUT AS #1
      INPUT " Enter the name of the file containing the interpolated values "; HH$
      PRINT
      OPEN HH$ FOR INPUT AS #3
      PRINT " Enter the name of a file to receive printer output "
      INPUT " (default is ROUT.DAT) "; Out$
      IF Out$ = "" THEN Out$ = "rout.dat"
      PRINT
      OPEN Out$ FOR OUTPUT AS #2
      PRINT " Enter the name of a file to receive the estimated totals"
      INPUT " (default is VALOUT.DAT) "; Zout$
      IF Zout$ = "" THEN Zout$ = "valout.dat"
      OPEN Zout$ FOR OUTPUT AS #4
      FMT$ = "##.###^^^^"
      FMT2$ = "####"
      PRINT
      LINE INPUT " Enter a title for the data "; Titl$
      PRINT #2, " "; Titl$
      PRINT
      INPUT " How many geographic zones are there "; NR
      INPUT " How many rows in the ID array "; RW%
      INPUT " How many columns in the ID array "; CL%
      INPUT " Do you wish the input array(s) listed "; IDOUT$
      IF IDOUT$ = "" THEN IDOUT$ = "N"
      IDOUT$ = UCASE$(MID$(IDOUT$, 1, 1))
      PRINT
      PRINT #2, NR; "zones"
      PRINT #2, RW%; "rows and"; CL%; "columns in the array"
      PRINT #2, " Resulting values will be sent to "; UCASE$(Zout$)
      DIM Z(RW%, CL%), ID(RW%, CL%)
      NR1 = NR + 1
      DIM Area(NR1), Valu(NR1), Dens(NR1), AA(NR1)
      RC% = RW% * CL%
      FOR K = 1 TO NR1
      Valu(K) = 0!
      Area(K) = 0!
      NEXT K
      ValSum = 0!
      IF IDOUT$ = "Y" THEN
        PRINT #2,
        PRINT #2, " Input values (from the "; UCASE$(HH$); " file):"
        PRINT #2,
      END IF
      PRINT " Reading Data Array";
      FOR I = 1 TO RW%
        PRINT ".";
        FOR J = 1 TO CL%
          INPUT #3, Z(I, J)
          ValSum = ValSum + Z(I, J)
        NEXT J
        IF IDOUT$ = "Y" THEN
          FOR KK = 1 TO CL%
            IF KK < CL% THEN
              PRINT #2, USING FMT$; Z(I, KK);
            ELSE
              PRINT #2, USING FMT$; Z(I, KK)
            END IF
          NEXT KK
        END IF
      NEXT I
      PRINT
      PRINT #2,
      IF IDOUT$ = "Y" THEN PRINT #2, " The ID array (from the "; UCASE$(ID$); " file):"
      PRINT " Reading ID Array and summing values";
      FOR I = 1 TO RW%
        PRINT ".";   'Just to watch progress
        FOR J = 1 TO CL%
          INPUT #1, ID(I, J)
          IF ID(I, J) <= 0 THEN ID(I, J) = NR1
          M = ID(I, J)
          Valu(M) = Valu(M) + Z(I, J)
          Area(M) = Area(M) + 1!
        NEXT J
        IF IDOUT$ = "Y" THEN
        FOR KK = 1 TO CL%
          IF KK < CL% THEN
            PRINT #2, USING FMT2$; ID(I, KK);
          ELSE
            PRINT #2, USING FMT2$; ID(I, KK)
          END IF
        NEXT KK
        END IF
      NEXT I
      PRINT
      PRINT " Done reading and summing."
      PRINT " Writing results."
      PRINT #2,
      PRINT #2, " Summing over the values in the "; UCASE$(HH$); " array"
      PRINT #2, " Using "; UCASE$(ID$); " as the indicator function."
      PRINT #2, " Values obtained by summation over the array:"
      PRINT #2, " ZONE NO.    VALUE         AREA         DENSITY"
      SumArea = 0!
      FOR M = 1 TO NR1
        Valu(M) = INT(Valu(M) + .5)
'       Valu is the count to distribute
'       Area is the area, Aa is the inverse area
        IF (Area(M) = 0!) THEN PRINT #2, M, Valu(M), Area(M); "Zero area"
        IF (Area(M) = 0!) THEN AA(M) = 0!
        IF Area(M) <> 0! THEN AA(M) = 1 / Area(M)
        Dens(M) = Valu(M) * AA(M)
        SumArea = SumArea + Area(M)
        IF (Area(M) <> 0!) THEN PRINT #2, M, Valu(M), Area(M), Dens(M)
      NEXT M
      SumArea = SumArea - Area(NR1)
      PRINT #2,
      PRINT #2, " Sum of data values:"; ValSum; "   Total area is"; SumArea; "of"; RC%; "units"
      ZNMAX = -1E+10
      ZNMIN = -ZNMAX
      FOR J = 1 TO NR1
        IF Valu(J) > ZNMAX THEN
          ZNMAX = Valu(J)
          JX = J
        END IF
        IF Valu(J) < ZNMIN THEN
          IF J = NR1 THEN GOTO SKIP
          ZNMIN = Valu(J)
          JN = J
SKIP:
        END IF
      NEXT J
      PRINT #2,
      PRINT #2, " Max  = "; ZNMAX; "   AT"; JX
      PRINT #2, " Min  = "; ZNMIN; "   AT"; JN
      PRINT #2, " Rows ="; RW%; ", Columns ="; CL%; ", Zones ="; NR
      ValSum = 0!
      FOR K = 1 TO NR1
        ValSum = ValSum + Valu(K)
        PRINT #4, K, Valu(K)
      NEXT K
      PRINT #4, " REGION      VALUE"
      PRINT #4,
      PRINT #4, " TOTAL = "; ValSum
      PRINT #4, " Values obtained by summation over the array."
      PRINT #4, " Summed over the values in the "; UCASE$(HH$); " array"
      PRINT #4, " Using "; UCASE$(ID$); " as the indicator function."
      PRINT #4, " ";
      PRINT #4, Titl$
      PRINT #2, " Resulting reaggregated values sent to "; UCASE$(Zout$)
      CLOSE
      PRINT
      PRINT " The program has completed its summation."
      PRINT " Look at the file "; UCASE$(Out$); " to see the results,"
      PRINT " and use the values in "; UCASE$(Zout$); " for further work."
      END

